@@ -15,12 +15,13 @@
 //
 // Copyright (C) 2006 Scott Turner <scotty1024@mac.com>
 // Copyright (C) 2007, 2008 Julien Rebetez <julienr@svn.gnome.org>
-// Copyright (C) 2007, 2008 Carlos Garcia Campos <carlosgc@gnome.org>
+// Copyright (C) 2007-2010 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright (C) 2007, 2008 Iñigo Martínez <inigomartinez@gmail.com>
 // Copyright (C) 2008 Michael Vrable <mvrable@cs.ucsd.edu>
 // Copyright (C) 2008 Hugo Mercier <hmercier31@gmail.com>
 // Copyright (C) 2008 Pino Toscano <pino@kde.org>
 // Copyright (C) 2008 Tomas Are Haavet <tomasare@gmail.com>
+// Copyright (C) 2009 Albert Astals Cid <aacid@kde.org>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -44,6 +45,7 @@ class Form;
 class FormWidget;
 class PDFRectangle;
 class Movie;
+class LinkAction;
 class OCGs;
 class Sound;
 
@@ -286,15 +288,17 @@ public:
   };
 
   AnnotColor();
-  AnnotColor(Array *array);
-  ~AnnotColor();
+  AnnotColor(double gray);
+  AnnotColor(double r, double g, double b);
+  AnnotColor(double c, double m, double y, double k);
+  AnnotColor(Array *array, int adjust = 0);
 
   AnnotColorSpace getSpace() const { return (AnnotColorSpace) length; }
-  double *getValues() const { return values; }
+  const double *getValues() const { return values; }
 
 private:
 
-  double *values;
+  double values[4];
   int length;
 };
 
@@ -479,7 +483,8 @@ public:
     type3D              // 3D             25
   };
 
-  Annot(XRef *xrefA, Dict *dict, Catalog* catalog);
+  Annot(XRef *xrefA, PDFRectangle *rectA, Catalog *catalog);
+  Annot(XRef *xrefA, Dict *dict, Catalog *catalog);
   Annot(XRef *xrefA, Dict *dict, Catalog *catalog, Object *obj);
   virtual ~Annot();
   GBool isOk() { return ok; }
@@ -496,11 +501,20 @@ public:
 
   double getFontSize() { return fontSize; }
 
+  // Sets the annot contents to new_content
+  // new_content should never be NULL
+  void setContents(GooString *new_content);
+
+  // The annotation takes the ownership of
+  // new_color. 
+  void setColor(AnnotColor *new_color);
+
   // getters
+  Ref getRef() const { return ref; }
   AnnotSubtype getType() const { return type; }
   PDFRectangle *getRect() const { return rect; }
   GooString *getContents() const { return contents; }
-  Dict *getPageDict() const { return pageDict; }
+  int getPageNum() const { return page; }
   GooString *getName() const { return name; }
   GooString *getModified() const { return modified; }
   Guint getFlags() const { return flags; }
@@ -509,7 +523,6 @@ public:
   AnnotBorder *getBorder() const { return border; }
   AnnotColor *getColor() const { return color; }
   int getTreeKey() const { return treeKey; }
-  OCGs *getOptionalContent() const { return optionalContent; }
 
   int getId() { return ref.num; }
 
@@ -521,10 +534,20 @@ private:
 
 
 protected:
-  void setColor(Array *a, GBool fill, int adjust);
+  void setColor(AnnotColor *color, GBool fill);
   void drawCircle(double cx, double cy, double r, GBool fill);
   void drawCircleTopLeft(double cx, double cy, double r);
   void drawCircleBottomRight(double cx, double cy, double r);
+  void createForm(double *bbox, GBool transparencyGroup, Object *resDict, Object *aStream);
+  void createResourcesDict(char *formName, Object *formStream, char *stateName,
+			   double opacity, char *blendMode, Object *resDict);
+  GBool isVisible(GBool printing);
+
+  // Updates the field key of the annotation dictionary
+  // and sets M to the current time
+  void update(const char *key, Object *value);
+
+  Object annotObj;
   
   // required data
   AnnotSubtype type;                // Annotation type
@@ -532,7 +555,7 @@ protected:
 
   // optional data
   GooString *contents;              // Contents
-  Dict *pageDict;                   // P
+  int       page;                   // P
   GooString *name;                  // NM
   GooString *modified;              // M
   Guint flags;                      // F (must be a 32 bit unsigned int)
@@ -542,7 +565,8 @@ protected:
                          //   for the normal appearance
   GooString *appearState;           // AS
   int treeKey;                      // Struct Parent;
-  OCGs *optionalContent;            // OC
+  OCGs *optContentConfig;           // Optional content config
+  Object oc;                        // OC
 
   XRef *xref;			// the xref table for this PDF file
   Ref ref;                      // object ref identifying this annotation
@@ -561,16 +585,21 @@ protected:
 
 class AnnotPopup: public Annot {
 public:
+  AnnotPopup(XRef *xrefA, PDFRectangle *rect, Catalog *catalog);
   AnnotPopup(XRef *xrefA, Dict *dict, Catalog *catalog, Object *obj);
   ~AnnotPopup();
 
-  Dict *getParent() const { return parent; }
+  Object *getParent(Object *obj) { return parent.fetch (xref, obj); }
+  Object *getParentNF(Object *obj) { return &parent; }
+  void setParent(Object *parentA);
+  void setParent(Annot *parentA);
   GBool getOpen() const { return open; }
+  void setOpen(GBool openA);
 
 protected:
   void initialize(XRef *xrefA, Dict *dict, Catalog *catalog);
 
-  Dict *parent; // Parent
+  Object parent; // Parent
   GBool open;   // Open
 };
 
@@ -585,6 +614,7 @@ public:
     replyTypeGroup  // Group
   };
 
+  AnnotMarkup(XRef *xrefA, PDFRectangle *rect, Catalog *catalog);
   AnnotMarkup(XRef *xrefA, Dict *dict, Catalog *catalog, Object *obj);
   virtual ~AnnotMarkup();
 
@@ -599,6 +629,10 @@ public:
   AnnotMarkupReplyType getReplyTo() const { return replyTo; }
   AnnotExternalDataType getExData() const { return exData; }
 
+  // The annotation takes the ownership of new_popup
+  void setPopup(AnnotPopup *new_popup);
+  void setLabel(GooString *new_label);
+
 protected:
   GooString *label;             // T            (Default autor)
   AnnotPopup *popup;            // Popup
@@ -636,16 +670,19 @@ public:
     stateNone       // None
   };
 
+  AnnotText(XRef *xrefA, PDFRectangle *rect, Catalog *catalog);
   AnnotText(XRef *xrefA, Dict *dict, Catalog *catalog, Object *obj);
   ~AnnotText();
 
+  virtual void draw(Gfx *gfx, GBool printing);
+
   // getters
   GBool getOpen() const { return open; }
   GooString *getIcon() const { return icon; }
   AnnotTextState getState() const { return state; }
 
-  // setters
-  void setModified(GooString *date);
+  void setOpen(GBool openA);
+  void setIcon(GooString *new_icon);
 
 private:
 
@@ -666,88 +703,20 @@ private:
 
 class AnnotMovie: public Annot {
  public:
-  enum PosterType {
-    posterTypeNone,
-    posterTypeStream,
-    posterTypeFromMovie
-  };
-
-  enum RepeatMode {
-    repeatModeOnce,
-    repeatModeOpen,
-    repeatModeRepeat,
-    repeatModePalindrome
-  };
-
-  struct Time {
-    Time() { units_per_second = 0; }
-    Gulong units;
-    int units_per_second; // 0 : defined by movie
-  };
-
+  AnnotMovie(XRef *xrefA, PDFRectangle *rect, Movie *movieA, Catalog *catalog);
   AnnotMovie(XRef *xrefA, Dict *dict, Catalog *catalog, Object *obj);
   ~AnnotMovie();
 
+  virtual void draw(Gfx *gfx, GBool printing);
+
   GooString* getTitle() { return title; }
-  GooString* getFileName() { return fileName; }
-  int getRotationAngle() { return rotationAngle; }
-
-  PosterType getPosterType() { return posterType; }
-  Stream* getPosterStream() { return posterStream; }
-
-  Time getStart() { return start; }
-  Time getDuration() { return duration; }
-  double getRate() { return rate; }
-  double getVolume() { return volume; }
-
-  GBool getShowControls() { return showControls; }
-  RepeatMode getRepeatMode() { return repeatMode; }
-  GBool getSynchronousPlay() { return synchronousPlay; }
-
-  GBool needFloatingWindow() { return hasFloatingWindow; }
-  GBool needFullscreen() { return isFullscreen; }
-  
-  
-  void getMovieSize(int& width, int& height);
-  void getZoomFactor(int& num, int& denum);
-  void getWindowPosition(double& x, double& y) { x = FWPosX; y = FWPosY; }
-
   Movie* getMovie() { return movie; }
 
  private:
   void initialize(XRef *xrefA, Catalog *catalog, Dict *dict);
 
   GooString* title;      // T
-  GooString* fileName;   // Movie/F
-
-  int width;             // Movie/Aspect
-  int height;            // Movie/Aspect
-  int rotationAngle;     // Movie/Rotate
-
-  PosterType posterType; // Movie/Poster
-  Stream* posterStream;
-
-  Time start;            // A/Start
-  Time duration;         // A/Duration
-  double rate;           // A/Rate
-  double volume;         // A/Volume
-  
-  GBool showControls;    // A/ShowControls
-  
-  RepeatMode repeatMode; // A/Mode
-  
-  GBool synchronousPlay; // A/Synchronous
-
-  // floating window
-  GBool hasFloatingWindow; 
-  unsigned short FWScaleNum; // A/FWScale
-  unsigned short FWScaleDenum;
-  GBool isFullscreen;
-
-  double FWPosX;         // A/FWPosition
-  double FWPosY; 
-
-  Movie* movie;
+  Movie* movie;          // Movie + A
 };
 
 
@@ -758,13 +727,14 @@ class AnnotMovie: public Annot {
 class AnnotScreen: public Annot {
  public:
 
+  AnnotScreen(XRef *xrefA, PDFRectangle *rect, Catalog *catalog);
   AnnotScreen(XRef *xrefA, Dict *dict, Catalog *catalog, Object *obj);
   ~AnnotScreen();
 
   GooString* getTitle() { return title; }
 
   AnnotAppearanceCharacs *getAppearCharacs() { return appearCharacs; }
-  Object* getAction() { return &action; }
+  LinkAction* getAction() { return action; }
   Object* getAdditionActions() { return &additionAction; }
 
  private:
@@ -775,7 +745,7 @@ class AnnotScreen: public Annot {
 
   AnnotAppearanceCharacs* appearCharacs; // MK
 
-  Object action;                         // A
+  LinkAction *action;                    // A
   Object additionAction;                 // AA
 };
 
@@ -793,6 +763,7 @@ public:
     effectPush      // P
   };
 
+  AnnotLink(XRef *xrefA, PDFRectangle *rect, Catalog *catalog);
   AnnotLink(XRef *xrefA, Dict *dict, Catalog *catalog, Object *obj);
   virtual ~AnnotLink();
 
@@ -836,6 +807,7 @@ public:
     intentFreeTextTypeWriter  // FreeTextTypeWriter
   };
 
+  AnnotFreeText(XRef *xrefA, PDFRectangle *rect, GooString *da, Catalog *catalog);
   AnnotFreeText(XRef *xrefA, Dict *dict, Catalog *catalog, Object *obj);
   ~AnnotFreeText();
 
@@ -887,9 +859,12 @@ public:
     captionPosTop     // Top
   };
 
+  AnnotLine(XRef *xrefA, PDFRectangle *rect, PDFRectangle *lRect, Catalog *catalog);
   AnnotLine(XRef *xrefA, Dict *dict, Catalog *catalog, Object *obj);
   ~AnnotLine();
 
+  virtual void draw(Gfx *gfx, GBool printing);
+
   // getters
   AnnotLineEndingStyle getStartStyle() const { return startStyle; }
   AnnotLineEndingStyle getEndStyle() const { return endStyle; }
@@ -939,9 +914,13 @@ protected:
 class AnnotTextMarkup: public AnnotMarkup {
 public:
 
+  AnnotTextMarkup(XRef *xrefA, PDFRectangle *rect, AnnotSubtype subType,
+		  AnnotQuadrilaterals *quadPoints, Catalog *catalog);
   AnnotTextMarkup(XRef *xrefA, Dict *dict, Catalog *catalog, Object *obj);
   virtual ~AnnotTextMarkup();
 
+  virtual void draw(Gfx *gfx, GBool printing);
+
   AnnotQuadrilaterals *getQuadrilaterals() const { return quadrilaterals; }
 
 protected:
@@ -958,6 +937,7 @@ protected:
 class AnnotStamp: public AnnotMarkup {
 public:
 
+  AnnotStamp(XRef *xrefA, PDFRectangle *rect, Catalog *catalog);
   AnnotStamp(XRef *xrefA, Dict *dict, Catalog *catalog, Object *obj);
   ~AnnotStamp();
 
@@ -978,9 +958,12 @@ private:
 class AnnotGeometry: public AnnotMarkup {
 public:
 
+  AnnotGeometry(XRef *xrefA, PDFRectangle *rect, AnnotSubtype subType, Catalog *catalog);
   AnnotGeometry(XRef *xrefA, Dict *dict, Catalog *catalog, Object *obj);
   ~AnnotGeometry();
 
+  virtual void draw(Gfx *gfx, GBool printing);
+
   // getters
   AnnotColor *getInteriorColor() const { return interiorColor; }
   AnnotBorderEffect *getBorderEffect() const { return borderEffect; }
@@ -1008,6 +991,8 @@ public:
     polygonDimension   // PolygonDimension
   };
 
+  AnnotPolygon(XRef *xrefA, PDFRectangle *rect, AnnotSubtype subType,
+	       AnnotPath *path, Catalog *catalog);
   AnnotPolygon(XRef *xrefA, Dict *dict, Catalog *catalog, Object *obj);
   ~AnnotPolygon();
 
@@ -1049,6 +1034,7 @@ public:
     symbolP         // P
   };
 
+  AnnotCaret(XRef *xrefA, PDFRectangle *rect, Catalog *catalog);
   AnnotCaret(XRef *xrefA, Dict *dict, Catalog *catalog, Object *obj);
   ~AnnotCaret();
 
@@ -1071,6 +1057,7 @@ private:
 class AnnotInk: public AnnotMarkup {
 public:
 
+  AnnotInk(XRef *xrefA, PDFRectangle *rect, AnnotPath **paths, int n_paths, Catalog *catalog);
   AnnotInk(XRef *xrefA, Dict *dict, Catalog *catalog, Object *obj);
   ~AnnotInk();
 
@@ -1098,9 +1085,12 @@ private:
 class AnnotFileAttachment: public AnnotMarkup {
 public:
 
+  AnnotFileAttachment(XRef *xrefA, PDFRectangle *rect, GooString *filename, Catalog *catalog);
   AnnotFileAttachment(XRef *xrefA, Dict *dict, Catalog *catalog, Object *obj);
   ~AnnotFileAttachment();
 
+  virtual void draw(Gfx *gfx, GBool printing);
+
   // getters
   Object *getFile() { return &file; }
   GooString *getName() const { return name; }
@@ -1123,9 +1113,12 @@ private:
 class AnnotSound: public AnnotMarkup {
 public:
 
+  AnnotSound(XRef *xrefA, PDFRectangle *rect, Sound *soundA, Catalog *catalog);
   AnnotSound(XRef *xrefA, Dict *dict, Catalog *catalog, Object *obj);
   ~AnnotSound();
 
+  virtual void draw(Gfx *gfx, GBool printing);
+
   // getters
   Sound *getSound() { return sound; }
   GooString *getName() const { return name; }
@@ -1194,6 +1187,7 @@ private:
   // AnnotBorderBS border;                // BS
   Dict *parent;                           // Parent
   GBool regen;
+  GBool addDingbatsResource;
 };
 
 //------------------------------------------------------------------------
@@ -1242,6 +1236,7 @@ class Annot3D: public Annot {
   };
 public:
 
+  Annot3D(XRef *xrefA, PDFRectangle *rect, Catalog *catalog);
   Annot3D(XRef *xrefA, Dict *dict, Catalog *catalog, Object *obj);
   ~Annot3D();
 
