@@ -33,7 +33,9 @@ typedef enum
 	POPPLER_ACTION_LAUNCH,		/* launch app (or open document) */
 	POPPLER_ACTION_URI,		/* URI */
 	POPPLER_ACTION_NAMED,		/* named action*/
-	POPPLER_ACTION_MOVIE		/* movie action */
+	POPPLER_ACTION_MOVIE,		/* movie action */
+	POPPLER_ACTION_RENDITION,       /* rendition action */
+	POPPLER_ACTION_OCG_STATE        /* Set-OCG-State action */
 } PopplerActionType;
 
 typedef enum
@@ -50,6 +52,21 @@ typedef enum
 	POPPLER_DEST_NAMED
 } PopplerDestType;
 
+typedef enum
+{
+        POPPLER_ACTION_MOVIE_PLAY,
+	POPPLER_ACTION_MOVIE_PAUSE,
+	POPPLER_ACTION_MOVIE_RESUME,
+	POPPLER_ACTION_MOVIE_STOP
+} PopplerActionMovieOperation;
+
+typedef enum
+{
+	POPPLER_ACTION_LAYER_ON,
+	POPPLER_ACTION_LAYER_OFF,
+	POPPLER_ACTION_LAYER_TOGGLE
+} PopplerActionLayerAction;
+
 /* Define the PopplerAction types */
 typedef struct _PopplerActionAny        PopplerActionAny;
 typedef struct _PopplerActionGotoDest   PopplerActionGotoDest;
@@ -58,6 +75,8 @@ typedef struct _PopplerActionLaunch     PopplerActionLaunch;
 typedef struct _PopplerActionUri        PopplerActionUri;
 typedef struct _PopplerActionNamed      PopplerActionNamed;
 typedef struct _PopplerActionMovie      PopplerActionMovie;
+typedef struct _PopplerActionRendition  PopplerActionRendition;
+typedef struct _PopplerActionOCGState   PopplerActionOCGState;
 
 struct _PopplerDest
 {
@@ -75,6 +94,11 @@ struct _PopplerDest
 	guint change_zoom : 1;
 };
 
+struct _PopplerActionLayer
+{
+	PopplerActionLayerAction action;
+	GList *layers;
+};
 
 struct _PopplerActionAny
 {
@@ -125,9 +149,29 @@ struct _PopplerActionNamed
 };
 
 struct _PopplerActionMovie
+{
+        PopplerActionType           type;
+        gchar                      *title;
+
+        PopplerActionMovieOperation operation;
+	PopplerMovie               *movie;
+};
+
+struct _PopplerActionRendition
 {
 	PopplerActionType type;
-	gchar *title;
+	gchar            *title;
+
+	gint               op;
+	PopplerMedia      *media;
+};
+
+struct _PopplerActionOCGState
+{
+	PopplerActionType type;
+	gchar            *title;
+
+	GList            *state_list;
 };
 
 union _PopplerAction
@@ -140,6 +184,8 @@ union _PopplerAction
 	PopplerActionUri uri;
 	PopplerActionNamed named;
 	PopplerActionMovie movie;
+	PopplerActionRendition rendition;
+	PopplerActionOCGState ocg_state;
 };
 
 #define POPPLER_TYPE_ACTION             (poppler_action_get_type ())
